import * as chalk from 'chalk';
import { decode } from '@rlnm/toolbelt';

import { Entry, parse } from './parse';

function pickColor(type: string, parents: string[]) {
	if (parents.indexOf('class') > -1) {
		switch (type) {
			case 'title':
				return chalk.yellow;
		}
	}

	if (parents.indexOf('meta') > -1) {
		switch (type) {
			case 'meta-string':
				return chalk.cyan;
			case 'keyword':
				return (chalk as any).redBright as (s: string) => string;
		}
	}

	switch (type) {
		case 'comment':
		case 'quote':
			return (chalk as any).cyanBright as (s: string) => string;
		case 'keyword':
		case 'selector-tag':
		case 'addition':
			return chalk.green;
		case 'number':
		case 'string':
		case 'literal':
		case 'doctag':
		case 'regexp':
			return chalk.cyan;
		case 'title':
		case 'section':
		case 'name':
		case 'selector-id':
		case 'selector-class':
			return chalk.blue;
		case 'attribute':
		case 'attr':
		case 'variable':
		case 'template-variable':
		case 'type':
			return chalk.yellow;
		case 'symbol':
		case 'bullet':
		case 'subst':
		case 'meta':
		case 'selector-attr':
		case 'selector-pseudo':
		case 'link':
			return (chalk as any).redBright as (s: string) => string;
		case 'built_in':
		case 'deletion':
			return chalk.red;
		case 'formula':
			return chalk.bgWhite;
		case 'emphasis':
			return chalk.italic;
		case 'strong':
			return chalk.bold;
		default:
			return (s: string) => s;
	}
}

export function highlightCli(source: string) {
	const stringifyNodes = (nodes: Entry[], parentTypes: string[]): string => {
		return nodes
			.map(node => {
				if (typeof node === 'string') {
					return decode(node);
				}

				return pickColor(node.type, parentTypes)(
					stringifyNodes(node.content, [...parentTypes, node.type])
				);
			})
			.join('');
	};

	return (chalk as any).yellowBright(stringifyNodes(parse(source), [])) as string;
}
