import { Entry, parse } from './parse';

function style(style: string) {
	return (content: string) => `<span style="${style}">${content}</span>`;
}

function pickColor(type: string, parents: string[]) {
	if (parents.indexOf('class') > -1) {
		switch (type) {
			case 'title':
				return style('color: #b58900');
		}
	}

	if (parents.indexOf('meta') > -1) {
		switch (type) {
			case 'meta-string':
				return style('color: #2aa198');
			case 'keyword':
				return style('color: #cb4b16');
		}
	}

	switch (type) {
		case 'comment':
		case 'quote':
			return style('color: #cb4b16');
		case 'keyword':
		case 'selector-tag':
		case 'addition':
			return style('color: #859900');
		case 'number':
		case 'string':
		case 'literal':
		case 'doctag':
		case 'regexp':
			return style('color: #2aa198');
		case 'title':
		case 'section':
		case 'name':
		case 'selector-id':
		case 'selector-class':
			return style('color: #268bd2');
		case 'attribute':
		case 'attr':
		case 'variable':
		case 'template-variable':
		case 'type':
			return style('color: #b58900');
		case 'symbol':
		case 'bullet':
		case 'subst':
		case 'meta':
		case 'selector-attr':
		case 'selector-pseudo':
		case 'link':
			return style('color: #cb4b16');
		case 'built_in':
		case 'deletion':
			return style('color: #dc322f');
		case 'formula':
			return style('color: #eee8d5');
		case 'emphasis':
			return style('font-style: italic');
		case 'strong':
			return style('font-weight: bold');
		default:
			return style('color: #657b83');
	}
}

export function highlightHtml(source: string) {
	const stringifyNodes = (nodes: Entry[], parentTypes: string[]): string => {
		return nodes
			.map(node => {
				if (typeof node === 'string') {
					return node;
				}

				return pickColor(node.type, parentTypes)(
					stringifyNodes(node.content, [...parentTypes, node.type])
				);
			})
			.join('');
	};

	return stringifyNodes(parse(source), []);
}
